@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -38,23 +36,31 @@
 #include "ctlib.h"
 #include "replacements.h"
 
-TDS_RCSID(var, "$Id: blk.c,v 1.53 2010/07/19 10:06:07 freddy77 Exp $");
-
 static void _blk_null_error(TDSBCPINFO *bcpinfo, int index, int offset);
-static int _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bcpcol, int offset);
+static TDSRET _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bcpcol, int offset);
 static CS_RETCODE _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred);
 static CS_RETCODE _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred);
 
+#define CONN(bulk) ((CS_CONNECTION *) (bulk)->bcpinfo.parent)
+
+TDS_COMPILE_CHECK(same_size, sizeof(CS_BLKDESC) == sizeof(TDSBCPINFO));
+TDS_COMPILE_CHECK(nested_type, TDS_OFFSET(CS_BLKDESC, bcpinfo) == 0);
+
 CS_RETCODE
 blk_alloc(CS_CONNECTION * connection, CS_INT version, CS_BLKDESC ** blk_pointer)
 {
+	CS_BLKDESC *blkdesc;
+
 	tdsdump_log(TDS_DBG_FUNC, "blk_alloc(%p, %d, %p)\n", connection, version, blk_pointer);
 
-	*blk_pointer = (CS_BLKDESC *) calloc(1, sizeof(CS_BLKDESC));
+	blkdesc = (CS_BLKDESC *) tds_alloc_bcpinfo();
+	if (!blkdesc)
+		return CS_FAIL;
 
 	/* so we know who we belong to */
-	(*blk_pointer)->con = connection;
+	blkdesc->bcpinfo.parent = connection;
 
+	*blk_pointer = blkdesc;
 	return CS_SUCCEED;
 }
 
@@ -72,7 +78,7 @@ blk_bind(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt, CS_VOID * buff
 	if (!blkdesc) {
 		return CS_FAIL;
 	}
-	con = blkdesc->con;
+	con = CONN(blkdesc);
 
 	if (item == CS_UNUSED) {
 		/* clear all bindings */
@@ -175,25 +181,22 @@ CS_RETCODE
 blk_describe(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt)
 {
 	TDSCOLUMN *curcol;
-	int len;
 
 	tdsdump_log(TDS_DBG_FUNC, "blk_describe(%p, %d, %p)\n", blkdesc, item, datafmt);
 
 	if (item < 1 || item > blkdesc->bcpinfo.bindinfo->num_cols) {
-		_ctclient_msg(blkdesc->con, "blk_describe", 2, 5, 1, 141, "%s, %d", "colnum", item);
+		_ctclient_msg(CONN(blkdesc), "blk_describe", 2, 5, 1, 141, "%s, %d", "colnum", item);
 		return CS_FAIL;
 	}
 
 	curcol = blkdesc->bcpinfo.bindinfo->columns[item - 1];
-	len = curcol->column_namelen;
-	if (len >= CS_MAX_NAME)
-		len = CS_MAX_NAME - 1;
-	strncpy(datafmt->name, curcol->column_name, len);
 	/* name is always null terminated */
-	datafmt->name[len] = 0;
-	datafmt->namelen = len;
+	strlcpy(datafmt->name, tds_dstr_cstr(&curcol->column_name), sizeof(datafmt->name));
+	datafmt->namelen = strlen(datafmt->name);
 	/* need to turn the SYBxxx into a CS_xxx_TYPE */
 	datafmt->datatype = _ct_get_client_type(curcol);
+	if (datafmt->datatype == CS_ILLEGAL_TYPE)
+		return CS_FAIL;
 	tdsdump_log(TDS_DBG_INFO1, "blk_describe() datafmt->datatype = %d server type %d\n", datafmt->datatype,
 			curcol->column_type);
 	/* FIXME is ok this value for numeric/decimal? */
@@ -226,27 +229,27 @@ blk_done(CS_BLKDESC * blkdesc, CS_INT type, CS_INT * outrow)
 
 	tdsdump_log(TDS_DBG_FUNC, "blk_done(%p, %d, %p)\n", blkdesc, type, outrow);
 
-	tds = blkdesc->con->tds_socket;
+	tds = CONN(blkdesc)->tds_socket;
 
 	switch (type) {
 	case CS_BLK_BATCH:
-		if (tds_bcp_done(tds, &rows_copied) != TDS_SUCCEED) {
-			_ctclient_msg(blkdesc->con, "blk_done", 2, 5, 1, 140, "");
+		if (TDS_FAILED(tds_bcp_done(tds, &rows_copied))) {
+			_ctclient_msg(CONN(blkdesc), "blk_done", 2, 5, 1, 140, "");
 			return CS_FAIL;
 		}
 		
 		if (outrow) 
 			*outrow = rows_copied;
 		
-		if (tds_bcp_start(tds, &blkdesc->bcpinfo) != TDS_SUCCEED) {
-			_ctclient_msg(blkdesc->con, "blk_done", 2, 5, 1, 140, "");
+		if (TDS_FAILED(tds_bcp_start(tds, &blkdesc->bcpinfo))) {
+			_ctclient_msg(CONN(blkdesc), "blk_done", 2, 5, 1, 140, "");
 			return CS_FAIL;
 		}
 		break;
 		
 	case CS_BLK_ALL:
-		if (tds_bcp_done(tds, &rows_copied) != TDS_SUCCEED) {
-			_ctclient_msg(blkdesc->con, "blk_done", 2, 5, 1, 140, "");
+		if (TDS_FAILED(tds_bcp_done(tds, &rows_copied))) {
+			_ctclient_msg(CONN(blkdesc), "blk_done", 2, 5, 1, 140, "");
 			return CS_FAIL;
 		}
 		
@@ -254,22 +257,11 @@ blk_done(CS_BLKDESC * blkdesc, CS_INT type, CS_INT * outrow)
 			*outrow = rows_copied;
 		
 		/* free allocated storage in blkdesc & initialise flags, etc. */
-	
-		if (blkdesc->bcpinfo.tablename)
-			TDS_ZERO_FREE(blkdesc->bcpinfo.tablename);
-	
-		if (blkdesc->bcpinfo.insert_stmt)
-			TDS_ZERO_FREE(blkdesc->bcpinfo.insert_stmt);
-	
-		if (blkdesc->bcpinfo.bindinfo) {
-			tds_free_results(blkdesc->bcpinfo.bindinfo);
-			blkdesc->bcpinfo.bindinfo = NULL;
-		}
+		tds_deinit_bcpinfo(&blkdesc->bcpinfo);
 	
 		blkdesc->bcpinfo.direction = 0;
 		blkdesc->bcpinfo.bind_count = CS_UNUSED;
 		blkdesc->bcpinfo.xfer_init = 0;
-		blkdesc->bcpinfo.var_cols = 0;
 
 		break;
 
@@ -283,13 +275,8 @@ blk_drop(CS_BLKDESC * blkdesc)
 {
 	tdsdump_log(TDS_DBG_FUNC, "blk_drop(%p)\n", blkdesc);
 
-	if (!blkdesc)
-		return CS_SUCCEED;
-
-	free(blkdesc->bcpinfo.tablename);
-	free(blkdesc->bcpinfo.insert_stmt);
-	tds_free_results(blkdesc->bcpinfo.bindinfo);
-	free(blkdesc);
+	/* this is possible as CS_BLKDESC contains just bcpinfo field */
+	tds_free_bcpinfo(&blkdesc->bcpinfo);
 
 	return CS_SUCCEED;
 }
@@ -323,48 +310,31 @@ blk_init(CS_BLKDESC * blkdesc, CS_INT direction, CS_CHAR * tablename, CS_INT tna
 	}
 
 	if (direction != CS_BLK_IN && direction != CS_BLK_OUT ) {
-		_ctclient_msg(blkdesc->con, "blk_init", 2, 6, 1, 138, "");
+		_ctclient_msg(CONN(blkdesc), "blk_init", 2, 6, 1, 138, "");
 		return CS_FAIL;
 	}
 
 	if (!tablename) {
-		_ctclient_msg(blkdesc->con, "blk_init", 2, 6, 1, 139, "");
+		_ctclient_msg(CONN(blkdesc), "blk_init", 2, 6, 1, 139, "");
 		return CS_FAIL;
 	}
 	if (tnamelen == CS_NULLTERM)
 		tnamelen = strlen(tablename);
 
 	/* free allocated storage in blkdesc & initialise flags, etc. */
-
-	if (blkdesc->bcpinfo.tablename) {
-		tdsdump_log(TDS_DBG_FUNC, "blk_init() freeing tablename\n");
-		free(blkdesc->bcpinfo.tablename);
-	}
-
-	if (blkdesc->bcpinfo.insert_stmt) {
-		tdsdump_log(TDS_DBG_FUNC, "blk_init() freeing insert_stmt\n");
-		TDS_ZERO_FREE(blkdesc->bcpinfo.insert_stmt);
-	}
-
-	if (blkdesc->bcpinfo.bindinfo) {
-		tdsdump_log(TDS_DBG_FUNC, "blk_init() freeing results\n");
-		tds_free_results(blkdesc->bcpinfo.bindinfo);
-		blkdesc->bcpinfo.bindinfo = NULL;
-	}
+	tds_deinit_bcpinfo(&blkdesc->bcpinfo);
 
 	/* string can be no-nul terminated so copy with memcpy */
-	blkdesc->bcpinfo.tablename = (char *) malloc(tnamelen + 1);
-	/* FIXME malloc can fail */
-	memcpy(blkdesc->bcpinfo.tablename, tablename, tnamelen);
-	blkdesc->bcpinfo.tablename[tnamelen] = 0;
+	if (!tds_dstr_copyn(&blkdesc->bcpinfo.tablename, tablename, tnamelen)) {
+		return CS_FAIL;
+	}
 
 	blkdesc->bcpinfo.direction = direction;
 	blkdesc->bcpinfo.bind_count = CS_UNUSED;
 	blkdesc->bcpinfo.xfer_init = 0;
-	blkdesc->bcpinfo.var_cols = 0;
 
-	if (tds_bcp_init(blkdesc->con->tds_socket, &blkdesc->bcpinfo) == TDS_FAIL) {
-		_ctclient_msg(blkdesc->con, "blk_init", 2, 5, 1, 140, "");
+	if (TDS_FAILED(tds_bcp_init(CONN(blkdesc)->tds_socket, &blkdesc->bcpinfo))) {
+		_ctclient_msg(CONN(blkdesc), "blk_init", 2, 5, 1, 140, "");
 		return CS_FAIL;
 	}
 	blkdesc->bcpinfo.bind_count = CS_UNUSED;
@@ -402,13 +372,13 @@ blk_props(CS_BLKDESC * blkdesc, CS_INT action, CS_INT property, CS_VOID * buffer
 			return CS_SUCCEED;
 			break;
 		default:
-			_ctclient_msg(blkdesc->con, "blk_props", 2, 5, 1, 141, "%s, %d", "action", action);
+			_ctclient_msg(CONN(blkdesc), "blk_props", 2, 5, 1, 141, "%s, %d", "action", action);
 			break;
 		}
 		break;
 
 	default:
-		_ctclient_msg(blkdesc->con, "blk_props", 2, 5, 1, 141, "%s, %d", "property", property);
+		_ctclient_msg(CONN(blkdesc), "blk_props", 2, 5, 1, 141, "%s, %d", "property", property);
 		break;
 	}
 	return CS_FAIL;
@@ -507,17 +477,15 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 {
 	TDSSOCKET *tds;
 	TDS_INT result_type;
-	TDS_INT ret;
+	TDSRET ret;
 	TDS_INT temp_count;
-	TDS_INT row_of_query;
-	TDS_INT rows_written;
 
 	tdsdump_log(TDS_DBG_FUNC, "_blk_rowxfer_out(%p, %d, %p)\n", blkdesc, rows_to_xfer, rows_xferred);
 
-	if (!blkdesc || !blkdesc->con)
+	if (!blkdesc || !CONN(blkdesc))
 		return CS_FAIL;
 
-	tds = blkdesc->con->tds_socket;
+	tds = CONN(blkdesc)->tds_socket;
 
 	/*
 	 * the first time blk_xfer called after blk_init()
@@ -526,28 +494,24 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 
 	if (blkdesc->bcpinfo.xfer_init == 0) {
 
-		if (tds_submit_queryf(tds, "select * from %s", blkdesc->bcpinfo.tablename)
-			== TDS_FAIL) {
-			_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 5, 1, 140, "");
+		if (TDS_FAILED(tds_submit_queryf(tds, "select * from %s", tds_dstr_cstr(&blkdesc->bcpinfo.tablename)))) {
+			_ctclient_msg(CONN(blkdesc), "blk_rowxfer", 2, 5, 1, 140, "");
 			return CS_FAIL;
 		}
-	
-		while ((ret = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
+
+		while ((ret = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCESS) {
 			if (result_type == TDS_ROW_RESULT)
 				break;
 		}
 	
-		if (ret != TDS_SUCCEED || result_type != TDS_ROW_RESULT) {
-			_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 5, 1, 140, "");
+		if (ret != TDS_SUCCESS || result_type != TDS_ROW_RESULT) {
+			_ctclient_msg(CONN(blkdesc), "blk_rowxfer", 2, 5, 1, 140, "");
 			return CS_FAIL;
 		}
 
 		blkdesc->bcpinfo.xfer_init = 1;
 	}
 
-	row_of_query = 0;
-	rows_written = 0;
-
 	if (rows_xferred)
 		*rows_xferred = 0;
 
@@ -558,10 +522,10 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 		tdsdump_log(TDS_DBG_FUNC, "blk_rowxfer_out() process_row_tokens returned %d\n", ret);
 
 		switch (ret) {
-		case TDS_SUCCEED:
+		case TDS_SUCCESS:
 			if (result_type == TDS_ROW_RESULT || result_type == TDS_COMPUTE_RESULT) {
 				if (result_type == TDS_ROW_RESULT) {
-					if (_ct_bind_data( blkdesc->con->ctx, tds->current_results, blkdesc->bcpinfo.bindinfo, temp_count))
+					if (_ct_bind_data( CONN(blkdesc)->ctx, tds->current_results, blkdesc->bcpinfo.bindinfo, temp_count))
 						return CS_ROW_FAIL;
 					if (rows_xferred)
 						*rows_xferred = *rows_xferred + 1;
@@ -573,7 +537,7 @@ _blk_rowxfer_out(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferre
 			break;
 
 		default:
-			_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 5, 1, 140, "");
+			_ctclient_msg(CONN(blkdesc), "blk_rowxfer", 2, 5, 1, 140, "");
 			return CS_FAIL;
 			break;
 		}
@@ -593,7 +557,7 @@ _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred
 	if (!blkdesc)
 		return CS_FAIL;
 
-	tds = blkdesc->con->tds_socket;
+	tds = CONN(blkdesc)->tds_socket;
 
 	/*
 	 * the first time blk_xfer called after blk_init()
@@ -607,19 +571,18 @@ _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred
 		 * retrieve details of the database table columns
 		 */
 
-		if (tds_bcp_start_copy_in(tds, &blkdesc->bcpinfo) == TDS_FAIL) {
-			_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 5, 1, 140, "");
+		if (TDS_FAILED(tds_bcp_start_copy_in(tds, &blkdesc->bcpinfo))) {
+			_ctclient_msg(CONN(blkdesc), "blk_rowxfer", 2, 5, 1, 140, "");
 			return CS_FAIL;
 		}
 
 		blkdesc->bcpinfo.xfer_init = 1;
 	} 
 
-	blkdesc->bcpinfo.parent = blkdesc;
 	for (each_row = 0; each_row < rows_to_xfer; each_row++ ) {
 
-		if (tds_bcp_send_record(tds, &blkdesc->bcpinfo, _blk_get_col_data, _blk_null_error, each_row) == TDS_SUCCEED) {
-	
+		if (tds_bcp_send_record(tds, &blkdesc->bcpinfo, _blk_get_col_data, _blk_null_error, each_row) == TDS_SUCCESS) {
+			/* FIXME */
 		}
 	}
 
@@ -629,14 +592,14 @@ _blk_rowxfer_in(CS_BLKDESC * blkdesc, CS_INT rows_to_xfer, CS_INT * rows_xferred
 static void
 _blk_null_error(TDSBCPINFO *bcpinfo, int index, int offset)
 {
-	CS_BLKDESC *blkdesc = (CS_BLKDESC *) bcpinfo->parent;
+	CS_BLKDESC *blkdesc = (CS_BLKDESC *) bcpinfo;
 
 	tdsdump_log(TDS_DBG_FUNC, "_blk_null_error(%p, %d, %d)\n", bcpinfo, index, offset);
 
-	_ctclient_msg(blkdesc->con, "blk_rowxfer", 2, 7, 1, 142, "%d, %d",  index + 1, offset + 1);
+	_ctclient_msg(CONN(blkdesc), "blk_rowxfer", 2, 7, 1, 142, "%d, %d",  index + 1, offset + 1);
 }
 
-static int
+static TDSRET
 _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 {
 	int result = 0;
@@ -648,8 +611,8 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 	CS_INT      destlen  = 0;
 	CS_SMALLINT *nullind = NULL;
 	CS_INT      *datalen = NULL;
-	CS_BLKDESC *blkdesc = (CS_BLKDESC *) bulk->parent;
-	CS_CONTEXT *ctx = blkdesc->con->ctx;
+	CS_BLKDESC *blkdesc = (CS_BLKDESC *) bulk;
+	CS_CONTEXT *ctx = CONN(blkdesc)->ctx;
 	CS_DATAFMT srcfmt, destfmt;
 
 	tdsdump_log(TDS_DBG_FUNC, "_blk_get_col_data(%p, %p, %d)\n", bulk, bindcol, offset);
@@ -660,6 +623,11 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 	 */
 
 	src = (unsigned char *) bindcol->column_varaddr;
+	if (!src) {
+		tdsdump_log(TDS_DBG_ERROR, "error source field not addressable\n");
+		return TDS_FAIL;
+	}
+
 	src += offset * bindcol->column_bindlen;
 	
 	if (bindcol->column_nullbind) {
@@ -671,68 +639,72 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 		datalen += offset;
 	}
 
-	if (src) {
-		srctype = bindcol->column_bindtype; 		/* passes to cs_convert */
+	srctype = bindcol->column_bindtype; 		/* passes to cs_convert */
 
-		tdsdump_log(TDS_DBG_INFO1, "blk_get_col_data srctype = %d \n", srctype);
-		tdsdump_log(TDS_DBG_INFO1, "blk_get_col_data datalen = %d \n", *datalen);
-	
-		if (*datalen) {
-			if (*datalen == CS_UNUSED) {
-				switch (srctype) {
-					case CS_LONG_TYPE:	    srclen = 8; break;
-					case CS_FLOAT_TYPE:	    srclen = 8; break;
-					case CS_MONEY_TYPE:	    srclen = 8; break;
-					case CS_DATETIME_TYPE:  srclen = 8; break;
-					case CS_INT_TYPE:	    srclen = 4; break;
-					case CS_REAL_TYPE:	    srclen = 4; break;
-					case CS_MONEY4_TYPE:	srclen = 4; break;
-					case CS_DATETIME4_TYPE: srclen = 4; break;
-					case CS_SMALLINT_TYPE:  srclen = 2; break;
-					case CS_TINYINT_TYPE:   srclen = 1; break;
-					default:
-						printf("error not fixed length type (%d) and datalen not specified\n",
-							bindcol->column_bindtype);
-						return CS_FAIL;
-				}
+	tdsdump_log(TDS_DBG_INFO1, "blk_get_col_data srctype = %d \n", srctype);
+	tdsdump_log(TDS_DBG_INFO1, "blk_get_col_data datalen = %d \n", *datalen);
 
-			} else {
-				srclen = *datalen;
-			}
-		}
-		if (srclen == 0) {
-			if (*nullind == -1) {
-				null_column = 1;
-			}
-		}
-
-		if (!null_column) {
-
-			srcfmt.datatype = srctype;
-			srcfmt.maxlength = srclen;
-
-			destfmt.datatype  = _ct_get_client_type(bindcol);
-			destfmt.maxlength = bindcol->column_size;
-			destfmt.precision = bindcol->column_prec;
-			destfmt.scale     = bindcol->column_scale;
-
-			destfmt.format	= CS_FMT_UNUSED;
-	
-			/* if convert return FAIL mark error but process other columns */
-			if ((result = cs_convert(ctx, &srcfmt, (CS_VOID *) src, 
-						 &destfmt, (CS_VOID *) bindcol->bcp_column_data->data, &destlen)) != CS_SUCCEED) {
-				tdsdump_log(TDS_DBG_INFO1, "convert failed for %d \n", srcfmt.datatype);
+	if (*datalen) {
+		if (*datalen == CS_UNUSED) {
+			switch (srctype) {
+			case CS_LONG_TYPE:	    srclen = 8; break;
+			case CS_FLOAT_TYPE:	    srclen = 8; break;
+			case CS_MONEY_TYPE:	    srclen = 8; break;
+			case CS_DATETIME_TYPE:  srclen = 8; break;
+			case CS_INT_TYPE:	    srclen = 4; break;
+			case CS_UINT_TYPE:	    srclen = 4; break;
+			case CS_REAL_TYPE:	    srclen = 4; break;
+			case CS_MONEY4_TYPE:	srclen = 4; break;
+			case CS_DATETIME4_TYPE: srclen = 4; break;
+			case CS_SMALLINT_TYPE:  srclen = 2; break;
+			case CS_USMALLINT_TYPE:  srclen = 2; break;
+			case CS_TINYINT_TYPE:   srclen = 1; break;
+			case CS_BIT_TYPE:   srclen = 1; break;
+			case CS_BIGINT_TYPE:	    srclen = 8; break;
+			case CS_UBIGINT_TYPE:	    srclen = 8; break;
+			case CS_UNIQUE_TYPE:	    srclen = 16; break;
+			default:
+				printf("error not fixed length type (%d) and datalen not specified\n",
+					bindcol->column_bindtype);
 				return CS_FAIL;
 			}
+
+		} else {
+			srclen = *datalen;
 		}
-
-		bindcol->bcp_column_data->datalen = destlen;
-		bindcol->bcp_column_data->is_null = null_column;
-
-		return CS_SUCCEED;
-
-	} else {
-		printf("error source field not addressable \n");
-		return CS_FAIL;
 	}
+	if (srclen == 0) {
+		if (nullind && *nullind == -1)
+			null_column = 1;
+	}
+
+	if (!null_column) {
+		CONV_RESULT convert_buffer;
+
+		srcfmt.datatype = srctype;
+		srcfmt.maxlength = srclen;
+
+		destfmt.datatype = _cs_convert_not_client(ctx, bindcol, &convert_buffer, &src);
+		if (destfmt.datatype == CS_ILLEGAL_TYPE)
+			destfmt.datatype  = _ct_get_client_type(bindcol);
+		if (destfmt.datatype == CS_ILLEGAL_TYPE)
+			return CS_FAIL;
+		destfmt.maxlength = bindcol->column_size;
+		destfmt.precision = bindcol->column_prec;
+		destfmt.scale     = bindcol->column_scale;
+
+		destfmt.format	= CS_FMT_UNUSED;
+
+		/* if convert return FAIL mark error but process other columns */
+		if ((result = cs_convert(ctx, &srcfmt, (CS_VOID *) src, 
+					 &destfmt, (CS_VOID *) bindcol->bcp_column_data->data, &destlen)) != CS_SUCCEED) {
+			tdsdump_log(TDS_DBG_INFO1, "convert failed for %d \n", srcfmt.datatype);
+			return CS_FAIL;
+		}
+	}
+
+	bindcol->bcp_column_data->datalen = destlen;
+	bindcol->bcp_column_data->is_null = null_column;
+
+	return TDS_SUCCESS;
 }
