@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005 Brian Bruns
+ * Copyright (C) 2015  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,24 +18,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <freetds/time.h>
 
 #include <stdio.h>
+#include <assert.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -46,55 +37,55 @@
 
 #include "cspublic.h"
 #include "ctlib.h"
-#include "tdsconvert.h"
 #include "replacements.h"
 
-TDS_RCSID(var, "$Id: cs.c,v 1.65 2007/12/23 21:12:02 jklowden Exp $");
+#undef cs_dt_crack
 
-static int _cs_datatype_length(int dtype);
 static CS_INT cs_diag_storemsg(CS_CONTEXT *context, CS_CLIENTMSG *message);
 static CS_INT cs_diag_clearmsg(CS_CONTEXT *context, CS_INT type);
 static CS_INT cs_diag_getmsg(CS_CONTEXT *context, CS_INT idx, CS_CLIENTMSG *message);
 static CS_INT cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count);
 
-/**
- * returns the fixed length of the specified data type, or 0 if not a 
- * fixed length data type
- */
-static int 
-_cs_datatype_length(int dtype)
+const char *
+cs_prretcode(int retcode)
 {
-	switch (dtype) {
-		case SYBINT1:
-			return 1;
-		case SYBINT2:
-			return 2;
-		case SYBINT4:
-			return 4;
-		case SYBINT8:
-			return 8;
-		case SYBFLT8:
-			return 8;
-		case SYBREAL:
-			return 4;
-		case SYBBIT:
-			return 1;
-		case SYBMONEY:
-			return 8;
-		case SYBMONEY4:
-			return 4;
-		case SYBDATETIME:
-			return 8;
-		case SYBDATETIME4:
-			return 4;
-		default:
-			return 0;
+	static char unknown[24];
+	
+	tdsdump_log(TDS_DBG_FUNC, "cs_prretcode(%d)\n", retcode);
+
+	switch(retcode) {
+	case CS_SUCCEED:	return "CS_SUCCEED";
+	case CS_FAIL:		return "CS_FAIL";
+	case CS_MEM_ERROR:	return "CS_MEM_ERROR";
+	case CS_PENDING:	return "CS_PENDING";
+	case CS_QUIET:		return "CS_QUIET";
+	case CS_BUSY:		return "CS_BUSY";
+	case CS_INTERRUPT:	return "CS_INTERRUPT";
+	case CS_BLK_HAS_TEXT:	return "CS_BLK_HAS_TEXT";
+	case CS_CONTINUE:	return "CS_CONTINUE";
+	case CS_FATAL:		return "CS_FATAL";
+	case CS_RET_HAFAILOVER:	return "CS_RET_HAFAILOVER";
+	case CS_UNSUPPORTED:	return "CS_UNSUPPORTED";
+
+	case CS_CANCELED:	return "CS_CANCELED";
+	case CS_ROW_FAIL:	return "CS_ROW_FAIL";
+	case CS_END_DATA:	return "CS_END_DATA";
+	case CS_END_RESULTS:	return "CS_END_RESULTS";
+	case CS_END_ITEM:	return "CS_END_ITEM";
+	case CS_NOMSG:		return "CS_NOMSG";
+	case CS_TIMED_OUT:	return "CS_TIMED_OUT";
+
+	default:
+		sprintf(unknown, "oops: %u ??", retcode);
 	}
+	return unknown;
 }
 
 static const char *
 _cs_get_layer(int layer)
 {
+	tdsdump_log(TDS_DBG_FUNC, "_cs_get_layer(%d)\n", layer);
+
 	switch (layer) {
 	case 2:
 		return "cslib user api layer";
@@ -108,6 +99,8 @@ _cs_get_layer(int layer)
 static const char *
 _cs_get_origin(int origin)
 {
+	tdsdump_log(TDS_DBG_FUNC, "_cs_get_origin(%d)\n", origin);
+
 	switch (origin) {
 	case 1:
 		return "external error";
@@ -130,6 +123,8 @@ _cs_get_origin(int origin)
 static const char *
 _cs_get_user_api_layer_error(int error)
 {
+	tdsdump_log(TDS_DBG_FUNC, "_cs_get_user_api_layer_error(%d)\n", error);
+
 	switch (error) {
 	case 3:
 		return "Memory allocation failure.";
@@ -154,6 +149,8 @@ _cs_get_msgstr(const char *funcname, int layer, int origin, int severity, int nu
 {
 	char *m;
 
+	tdsdump_log(TDS_DBG_FUNC, "_cs_get_msgstr(%s, %d, %d, %d, %d)\n", funcname, layer, origin, severity, number);
+
 	if (asprintf(&m, "%s: %s: %s: %s", funcname, _cs_get_layer(layer), _cs_get_origin(origin), (layer == 2)
 		     ? _cs_get_user_api_layer_error(number)
 		     : "unrecognized error") < 0) {
@@ -169,6 +166,8 @@ _csclient_msg(CS_CONTEXT * ctx, const char *funcname, int layer, int origin, int
 	CS_CLIENTMSG cm;
 	char *msgstr;
 
+	tdsdump_log(TDS_DBG_FUNC, "_csclient_msg(%p, %s, %d, %d, %d, %d, %s)\n", ctx, funcname, layer, origin, severity, number, fmt);
+
 	va_start(ap, fmt);
 
 	if (ctx->_cslibmsg_cb) {
@@ -200,12 +199,16 @@ _csclient_msg(CS_CONTEXT * ctx, const char *funcname, int layer, int origin, int
 static CS_LOCALE *
 _cs_locale_alloc(void)
 {
-	return (CS_LOCALE *) calloc(1, sizeof(CS_LOCALE));
+	tdsdump_log(TDS_DBG_FUNC, "_cs_locale_alloc()\n");
+
+	return tds_new0(CS_LOCALE, 1);
 }
 
 static void
 _cs_locale_free_contents(CS_LOCALE *locale)
 {
+	tdsdump_log(TDS_DBG_FUNC, "_cs_locale_free_contents(%p)\n", locale);
+
 	/* free strings */
 	free(locale->language);
 	locale->language = NULL;
@@ -220,6 +223,8 @@ _cs_locale_free_contents(CS_LOCALE *locale)
 void 
 _cs_locale_free(CS_LOCALE *locale)
 {
+	tdsdump_log(TDS_DBG_FUNC, "_cs_locale_free(%p)\n", locale);
+
 	/* free contents */
 	_cs_locale_free_contents(locale);
 
@@ -231,6 +236,8 @@ _cs_locale_free(CS_LOCALE *locale)
 int
 _cs_locale_copy_inplace(CS_LOCALE *new_locale, CS_LOCALE *orig)
 {
+	tdsdump_log(TDS_DBG_FUNC, "_cs_locale_copy_inplace(%p, %p)\n", new_locale, orig);
+
 	_cs_locale_free_contents(new_locale);
 	if (orig->language) {
 		new_locale->language = strdup(orig->language);
@@ -269,6 +276,8 @@ _cs_locale_copy(CS_LOCALE *orig)
 {
 	CS_LOCALE *new_locale;
 
+	tdsdump_log(TDS_DBG_FUNC, "_cs_locale_copy(%p)\n", orig);
+
 	new_locale = _cs_locale_alloc();
 	if (!new_locale)
 		return NULL;
@@ -304,15 +313,14 @@ Cleanup:
 	return NULL;
 }
 
-extern const char STD_DATETIME_FMT[];
-
 CS_RETCODE
 cs_ctx_alloc(CS_INT version, CS_CONTEXT ** ctx)
 {
-TDSCONTEXT *tds_ctx;
+	TDSCONTEXT *tds_ctx;
 
-	*ctx = (CS_CONTEXT *) malloc(sizeof(CS_CONTEXT));
-	memset(*ctx, '\0', sizeof(CS_CONTEXT));
+	tdsdump_log(TDS_DBG_FUNC, "cs_ctx_alloc(%d, %p)\n", version, ctx);
+
+	*ctx = tds_new0(CS_CONTEXT, 1);
 	tds_ctx = tds_alloc_context(*ctx);
 	if (!tds_ctx) {
 		free(*ctx);
@@ -329,7 +337,9 @@ TDSCONTEXT *tds_ctx;
 CS_RETCODE
 cs_ctx_global(CS_INT version, CS_CONTEXT ** ctx)
 {
-static CS_CONTEXT *global_cs_ctx = NULL;
+	static CS_CONTEXT *global_cs_ctx = NULL;
+
+	tdsdump_log(TDS_DBG_FUNC, "cs_ctx_global(%d, %p)\n", version, ctx);
 
 	if (global_cs_ctx != NULL) {
 		*ctx = global_cs_ctx;
@@ -345,6 +355,8 @@ static CS_CONTEXT *global_cs_ctx = NULL;
 CS_RETCODE
 cs_ctx_drop(CS_CONTEXT * ctx)
 {
+	tdsdump_log(TDS_DBG_FUNC, "cs_ctx_drop(%p)\n", ctx);
+
 	if (ctx) {
 		_ct_diag_clearmsg(ctx, CS_ALLMSG_TYPE);
 		free(ctx->userdata);
@@ -358,8 +370,9 @@ cs_ctx_drop(CS_CONTEXT * ctx)
 CS_RETCODE
 cs_config(CS_CONTEXT * ctx, CS_INT action, CS_INT property, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen)
 {
-/* declared  for - CS_USERDATA changes - swapna*/
-CS_INT maxcp;
+	CS_INT maxcp;	/* declared  for - CS_USERDATA changes - swapna*/
+
+	tdsdump_log(TDS_DBG_FUNC, "cs_config(%p, %d, %d, %p, %d, %p)\n", ctx, action, property, buffer, buflen, outlen);
 
 	if (action == CS_GET) {
 		if (buffer == NULL) {
@@ -399,7 +412,7 @@ CS_INT maxcp;
 			free(ctx->userdata);
 	
 			if (buflen == CS_NULLTERM) {
-				maxcp = strlen(buffer) + 1;
+				maxcp = strlen((char*) buffer) + 1;
 			} else {
 				maxcp = buflen;
 			}
@@ -451,23 +464,67 @@ CS_INT maxcp;
 CS_RETCODE
 cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT * destfmt, CS_VOID * destdata, CS_INT * resultlen)
 {
+	TDS_SERVER_TYPE src_type, desttype;
+	int src_len, destlen, len, i = 0;
+	CONV_RESULT cres;
+	unsigned char *dest;
+	CS_RETCODE ret;
+	CS_INT dummy;
+	CS_VARCHAR *destvc = NULL;
 
-int src_type, src_len, desttype, destlen, len, i = 0;
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert(%p, %p, %p, %p, %p, %p)\n", ctx, srcfmt, srcdata, destfmt, destdata, resultlen);
 
-CONV_RESULT cres;
+	/* If destination is NULL we have a problem */
+	if (destdata == NULL) {
+		/* TODO: add error message */
+		tdsdump_log(TDS_DBG_FUNC, "error: destdata is null\n");
+		return CS_FAIL;
 
-unsigned char *dest;
+	}
 
-CS_RETCODE ret;
+	/* If destfmt is NULL we have a problem */
+	if (destfmt == NULL) {
+		/* TODO: add error message */
+		tdsdump_log(TDS_DBG_FUNC, "error: destfmt is null\n");
+		return CS_FAIL;
 
-	tdsdump_log(TDS_DBG_FUNC, "cs_convert()\n");
+	}
+	
+	if (resultlen == NULL) 
+		resultlen = &dummy;
 
-	src_type = _ct_get_server_type(srcfmt->datatype);
-	src_len = srcfmt ? srcfmt->maxlength : 0;
-	desttype = _ct_get_server_type(destfmt->datatype);
-	destlen = destfmt ? destfmt->maxlength : 0;
+	/* If source is indicated to be NULL, set dest to low values */
+	if (srcdata == NULL) {
+		/* TODO: implement cs_setnull */
+		tdsdump_log(TDS_DBG_FUNC, "srcdata is null\n");
+		memset(destdata, '\0', destfmt->maxlength);
+		*resultlen = 0;
+		return CS_SUCCEED;
 
-	tdsdump_log(TDS_DBG_FUNC, "cs_convert() srctype = %d (%d) desttype = %d (%d)\n",
+	}
+
+	src_type = _ct_get_server_type(NULL, srcfmt->datatype);
+	if (src_type == TDS_INVALID_TYPE)
+		return CS_FAIL;
+	src_len = srcfmt->maxlength;
+	if (srcfmt->datatype == CS_VARCHAR_TYPE || srcfmt->datatype == CS_VARBINARY_TYPE) {
+		CS_VARCHAR *vc = (CS_VARCHAR *) srcdata;
+		src_len = vc->len;
+		srcdata = vc->str;
+	}
+	desttype = _ct_get_server_type(NULL, destfmt->datatype);
+	if (desttype == TDS_INVALID_TYPE)
+		return CS_FAIL;
+	destlen = destfmt->maxlength;
+	if (destfmt->datatype == CS_VARCHAR_TYPE || destfmt->datatype == CS_VARBINARY_TYPE) {
+		destvc = (CS_VARCHAR *) destdata;
+		destlen  = sizeof(destvc->str);
+		destdata = destvc->str;
+	} else if (is_numeric_type(desttype)) {
+		destlen = sizeof(TDS_NUMERIC);
+	}
+
+	tdsdump_log(TDS_DBG_FUNC, "converting type %d (%d bytes) to type = %d (%d bytes)\n",
 		    src_type, src_len, desttype, destlen);
 
 	if (!is_fixed_type(desttype) && (destlen <= 0)) {
@@ -476,98 +533,80 @@ CS_RETCODE ret;
 
 	dest = (unsigned char *) destdata;
 
-	/* If source is indicated to be NULL, set dest to low values */
-
-	if (srcdata == NULL) {
-
-		tdsdump_log(TDS_DBG_FUNC, "cs_convert() srcdata is null\n");
-		memset(dest, '\0', destlen);
-		if (resultlen != NULL)
-			*resultlen = 0;
-		return CS_SUCCEED;
-
-	}
-
 	/* many times we are asked to convert a data type to itself */
 
 	if (src_type == desttype) {
+		int minlen = src_len < destlen? src_len : destlen;
 
-		tdsdump_log(TDS_DBG_FUNC, "cs_convert() srctype = desttype\n");
+		tdsdump_log(TDS_DBG_FUNC, "cs_convert() srctype == desttype\n");
 		switch (desttype) {
 
 		case SYBLONGBINARY:
 		case SYBBINARY:
 		case SYBVARBINARY:
 		case SYBIMAGE:
+			memcpy(dest, srcdata, src_len);
+			*resultlen = src_len;
+
 			if (src_len > destlen) {
+				tdsdump_log(TDS_DBG_FUNC, "error: src_len > destlen\n");
 				ret = CS_FAIL;
 			} else {
 				switch (destfmt->format) {
 				case CS_FMT_PADNULL:
-					memcpy(dest, srcdata, src_len);
-					for (i = src_len; i < destlen; i++)
-						dest[i] = '\0';
-					if (resultlen != NULL)
-						*resultlen = destlen;
-					ret = CS_SUCCEED;
-					break;
+					memset(dest + src_len, '\0', destlen - src_len);
+					*resultlen = destlen;
+					/* fall through */
 				case CS_FMT_UNUSED:
-					memcpy(dest, srcdata, src_len);
-					if (resultlen != NULL)
-						*resultlen = src_len;
 					ret = CS_SUCCEED;
 					break;
 				default:
 					ret = CS_FAIL;
 					break;
-
 				}
 			}
+			if (destvc) {
+				destvc->len = minlen;
+				*resultlen = sizeof(*destvc);
+			}
 			break;
 
 		case SYBCHAR:
 		case SYBVARCHAR:
 		case SYBTEXT:
 			tdsdump_log(TDS_DBG_FUNC, "cs_convert() desttype = character\n");
+
+			memcpy(dest, srcdata, minlen);
+			*resultlen = minlen;
+
 			if (src_len > destlen) {
-				tdsdump_log(TDS_DBG_FUNC, "src_len > destlen\n");
+				tdsdump_log(TDS_DBG_FUNC, "error: src_len > destlen\n");
 				ret = CS_FAIL;
 			} else {
 				switch (destfmt->format) {
-
 				case CS_FMT_NULLTERM:
 					if (src_len == destlen) {
-						ret = CS_FAIL;	/* not enough room for data + a null terminator - error */
+						*resultlen = src_len;
+						tdsdump_log(TDS_DBG_FUNC, "error: no room for null terminator\n");
+						ret = CS_FAIL;
 					} else {
-						memcpy(dest, srcdata, src_len);
 						dest[src_len] = '\0';
-						if (resultlen != NULL)
-							*resultlen = src_len + 1;
+						*resultlen = src_len + 1;
 						ret = CS_SUCCEED;
 					}
 					break;
 
 				case CS_FMT_PADBLANK:
-					memcpy(dest, srcdata, src_len);
-					for (i = src_len; i < destlen; i++)
-						dest[i] = ' ';
-					if (resultlen != NULL)
-						*resultlen = destlen;
+					memset(dest + src_len, ' ', destlen - src_len);
+					*resultlen = destlen;
 					ret = CS_SUCCEED;
 					break;
-
 				case CS_FMT_PADNULL:
-					memcpy(dest, srcdata, src_len);
-					for (i = src_len; i < destlen; i++)
-						dest[i] = '\0';
-					if (resultlen != NULL)
-						*resultlen = destlen;
+					memset(dest + src_len, '\0', destlen - src_len);
+					*resultlen = destlen;
 					ret = CS_SUCCEED;
 					break;
 				case CS_FMT_UNUSED:
-					memcpy(dest, srcdata, src_len);
-					if (resultlen != NULL)
-						*resultlen = src_len;
 					ret = CS_SUCCEED;
 					break;
 				default:
@@ -576,11 +615,19 @@ CS_RETCODE ret;
 					break;
 				}
 			}
+			if (destvc) {
+				destvc->len = minlen;
+				*resultlen = sizeof(*destvc);
+			}
 			break;
 		case SYBINT1:
+		case SYBUINT1:
 		case SYBINT2:
+		case SYBUINT2:
 		case SYBINT4:
+		case SYBUINT4:
 		case SYBINT8:
+		case SYBUINT8:
 		case SYBFLT8:
 		case SYBREAL:
 		case SYBBIT:
@@ -588,9 +635,13 @@ CS_RETCODE ret;
 		case SYBMONEY4:
 		case SYBDATETIME:
 		case SYBDATETIME4:
-			memcpy(dest, srcdata, _cs_datatype_length(src_type));
-			if (resultlen != NULL)
-				*resultlen = _cs_datatype_length(src_type);
+		case SYBTIME:
+		case SYBDATE:
+		case SYB5BIGDATETIME:
+		case SYB5BIGTIME:
+			*resultlen = tds_get_size_by_type(src_type);
+			if (*resultlen > 0)
+				memcpy(dest, srcdata, *resultlen);
 			ret = CS_SUCCEED;
 			break;
 
@@ -598,27 +649,31 @@ CS_RETCODE ret;
 		case SYBDECIMAL:
 			src_len = tds_numeric_bytes_per_prec[((TDS_NUMERIC *) srcdata)->precision] + 2;
 		case SYBBITN:
+		case SYBUNIQUE:
+			memcpy(dest, srcdata, minlen);
+			*resultlen = minlen;
+
 			if (src_len > destlen) {
+				tdsdump_log(TDS_DBG_FUNC, "error: src_len > destlen\n");
 				ret = CS_FAIL;
 			} else {
-				memcpy(dest, srcdata, src_len);
-				if (resultlen != NULL)
-					*resultlen = src_len;
 				ret = CS_SUCCEED;
 			}
 			break;
 
 		default:
+			tdsdump_log(TDS_DBG_FUNC, "error: unrecognized type\n");
 			ret = CS_FAIL;
 			break;
 		}
 
+		tdsdump_log(TDS_DBG_FUNC, "cs_convert() returning  %s\n", cs_prretcode(ret));
 		return ret;
 
 	}
 
-
-	/* src type != dest type */
+	assert(src_type != desttype);
+	
 	/* set the output precision/scale for conversions to numeric type */
 	if (is_numeric_type(desttype)) {
 		cres.n.precision = destfmt->precision;
@@ -665,149 +720,107 @@ CS_RETCODE ret;
 	case SYBBINARY:
 	case SYBVARBINARY:
 	case SYBIMAGE:
-
+		ret = CS_SUCCEED;
 		if (len > destlen) {
-			free(cres.ib);
-			fprintf(stderr, "error_handler: Data-conversion resulted in overflow.\n");
+			tdsdump_log(TDS_DBG_FUNC, "error_handler: Data-conversion resulted in overflow\n");
 			ret = CS_FAIL;
-		} else {
-			memcpy(dest, cres.ib, len);
-			free(cres.ib);
-			for (i = len; i < destlen; i++)
-				dest[i] = '\0';
-			if (resultlen != NULL)
-				*resultlen = destlen;
-			ret = CS_SUCCEED;
+			len = destlen;
 		}
+		memcpy(dest, cres.ib, len);
+		free(cres.ib);
+		*resultlen = destlen;
+		if (destvc) {
+			destvc->len = len;
+			*resultlen = sizeof(*destvc);
+		}
+		for (i = len; i < destlen; i++)
+			dest[i] = '\0';
 		break;
 	case SYBBIT:
 	case SYBBITN:
 		/* fall trough, act same way of TINYINT */
 	case SYBINT1:
-		memcpy(dest, &(cres.ti), 1);
-		if (resultlen != NULL)
-			*resultlen = 1;
-		ret = CS_SUCCEED;
-		break;
+	case SYBUINT1:
 	case SYBINT2:
-		memcpy(dest, &(cres.si), 2);
-		if (resultlen != NULL)
-			*resultlen = 2;
-		ret = CS_SUCCEED;
-		break;
+	case SYBUINT2:
 	case SYBINT4:
-		memcpy(dest, &(cres.i), 4);
-		if (resultlen != NULL)
-			*resultlen = 4;
-		ret = CS_SUCCEED;
-		break;
+	case SYBUINT4:
 	case SYBINT8:
-		memcpy(dest, &(cres.bi), 8);
-		if (resultlen != NULL)
-			*resultlen = 8;
-		ret = CS_SUCCEED;
-		break;
+	case SYBUINT8:
 	case SYBFLT8:
-		memcpy(dest, &(cres.f), 8);
-		if (resultlen != NULL)
-			*resultlen = 8;
-		ret = CS_SUCCEED;
-		break;
 	case SYBREAL:
-		memcpy(dest, &(cres.r), 4);
-		if (resultlen != NULL)
-			*resultlen = 4;
-		ret = CS_SUCCEED;
-		break;
 	case SYBMONEY:
-
-		tdsdump_log(TDS_DBG_FUNC, "cs_convert() copying %d bytes to src\n", (int) sizeof(TDS_MONEY));
-		memcpy(dest, &(cres.m), sizeof(TDS_MONEY));
-		if (resultlen != NULL)
-			*resultlen = sizeof(TDS_MONEY);
-		ret = CS_SUCCEED;
-		break;
 	case SYBMONEY4:
-		memcpy(dest, &(cres.m4), sizeof(TDS_MONEY4));
-		if (resultlen != NULL)
-			*resultlen = sizeof(TDS_MONEY4);
-		ret = CS_SUCCEED;
-		break;
 	case SYBDATETIME:
-		memcpy(dest, &(cres.dt), sizeof(TDS_DATETIME));
-		if (resultlen != NULL)
-			*resultlen = sizeof(TDS_DATETIME);
-		ret = CS_SUCCEED;
-		break;
 	case SYBDATETIME4:
-		memcpy(dest, &(cres.dt4), sizeof(TDS_DATETIME4));
-		if (resultlen != NULL)
-			*resultlen = sizeof(TDS_DATETIME4);
+	case SYBTIME:
+	case SYBDATE:
+	case SYBUNIQUE:
+	case SYB5BIGDATETIME:
+	case SYB5BIGTIME:
+		*resultlen = tds_get_size_by_type(desttype);
+		memcpy(dest, &(cres.ti), *resultlen);
 		ret = CS_SUCCEED;
 		break;
 	case SYBNUMERIC:
 	case SYBDECIMAL:
 		src_len = tds_numeric_bytes_per_prec[cres.n.precision] + 2;
 		memcpy(dest, &(cres.n), src_len);
-		if (resultlen != NULL)
-			*resultlen = src_len;
+		*resultlen = src_len;
 		ret = CS_SUCCEED;
 		break;
 	case SYBCHAR:
 	case SYBVARCHAR:
 	case SYBTEXT:
+		ret = CS_SUCCEED;
 		if (len > destlen) {
-			fprintf(stderr, "error_handler: Data-conversion resulted in overflow.\n");
+			tdsdump_log(TDS_DBG_FUNC, "Data-conversion resulted in overflow\n");
+			len = destlen;
 			ret = CS_FAIL;
-		} else {
-			switch (destfmt->format) {
+		}
+		switch (destfmt->format) {
 
-			case CS_FMT_NULLTERM:
-				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_NULLTERM\n");
-				if (len == destlen) {
-					tdsdump_log(TDS_DBG_FUNC, "not enough room for data + a null terminator - error\n");
-					ret = CS_FAIL;	/* not enough room for data + a null terminator - error */
-				} else {
-					memcpy(dest, cres.c, len);
-					dest[len] = 0;
-					if (resultlen != NULL)
-						*resultlen = len + 1;
-					ret = CS_SUCCEED;
-				}
-				break;
-
-			case CS_FMT_PADBLANK:
-				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADBLANK\n");
-				/* strcpy here can lead to a small buffer overflow */
+		case CS_FMT_NULLTERM:
+			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_NULLTERM\n");
+			if (len == destlen) {
+				tdsdump_log(TDS_DBG_FUNC, "not enough room for data + a null terminator - error\n");
+				ret = CS_FAIL;	/* not enough room for data + a null terminator - error */
+			} else {
 				memcpy(dest, cres.c, len);
-				for (i = len; i < destlen; i++)
-					dest[i] = ' ';
-				if (resultlen != NULL)
-					*resultlen = destlen;
-				ret = CS_SUCCEED;
-				break;
-
-			case CS_FMT_PADNULL:
-				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADNULL\n");
-				/* strcpy here can lead to a small buffer overflow */
-				memcpy(dest, cres.c, len);
-				for (i = len; i < destlen; i++)
-					dest[i] = '\0';
-				if (resultlen != NULL)
-					*resultlen = destlen;
-				ret = CS_SUCCEED;
-				break;
-			case CS_FMT_UNUSED:
-				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_UNUSED\n");
-				memcpy(dest, cres.c, len);
-				if (resultlen != NULL)
-					*resultlen = len;
-				ret = CS_SUCCEED;
-				break;
-			default:
-				ret = CS_FAIL;
-				break;
+				dest[len] = 0;
+				*resultlen = len + 1;
 			}
+			break;
+
+		case CS_FMT_PADBLANK:
+			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADBLANK\n");
+			/* strcpy here can lead to a small buffer overflow */
+			memcpy(dest, cres.c, len);
+			for (i = len; i < destlen; i++)
+				dest[i] = ' ';
+			*resultlen = destlen;
+			break;
+
+		case CS_FMT_PADNULL:
+			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADNULL\n");
+			/* strcpy here can lead to a small buffer overflow */
+			memcpy(dest, cres.c, len);
+			for (i = len; i < destlen; i++)
+				dest[i] = '\0';
+			*resultlen = destlen;
+			break;
+		case CS_FMT_UNUSED:
+			tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_UNUSED\n");
+			memcpy(dest, cres.c, len);
+			*resultlen = len;
+			break;
+		default:
+			ret = CS_FAIL;
+			break;
+		}
+		if (destvc) {
+			destvc->len = len;
+			*resultlen = sizeof(*destvc);
 		}
 		free(cres.c);
 		break;
@@ -815,41 +828,74 @@ CS_RETCODE ret;
 		ret = CS_FAIL;
 		break;
 	}
-	tdsdump_log(TDS_DBG_FUNC, "cs_convert() returning  %d\n", ret);
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert() returning  %s\n", cs_prretcode(ret));
 	return (ret);
 }
 
+CS_RETCODE
+cs_dt_crack_v2(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
+{
+	TDSDATEREC dr;
+	TDS_INT tds_type;
+	bool extended = false;
+
+	tdsdump_log(TDS_DBG_FUNC, "cs_dt_crack_v2(%p, %d, %p, %p)\n", ctx, datetype, dateval, daterec);
+
+	switch (datetype) {
+	case CS_DATETIME_TYPE:
+		tds_type = SYBDATETIME;
+		break;
+	case CS_DATETIME4_TYPE:
+		tds_type = SYBDATETIME4;
+		break;
+	case CS_DATE_TYPE:
+		tds_type = SYBDATE;
+		break;
+	case CS_TIME_TYPE:
+		tds_type = SYBTIME;
+		break;
+	case CS_BIGDATETIME_TYPE:
+		tds_type = SYB5BIGDATETIME;
+		extended = true;
+		break;
+	case CS_BIGTIME_TYPE:
+		tds_type = SYB5BIGTIME;
+		extended = true;
+		break;
+	default:
+		return CS_FAIL;
+	}
+	tds_datecrack(tds_type, dateval, &dr);
+
+	/* Sybase CT-Library does not set these fields for CS_BIGTIME_TYPE */
+	if (tds_type != SYB5BIGTIME) {
+		daterec->dateyear = dr.year;
+		daterec->datemonth = dr.month;
+		daterec->datedmonth = dr.day;
+		daterec->datedyear = dr.dayofyear;
+		daterec->datedweek = dr.weekday;
+	}
+	daterec->datehour = dr.hour;
+	daterec->dateminute = dr.minute;
+	daterec->datesecond = dr.second;
+	daterec->datemsecond = dr.decimicrosecond / 10000u;
+	daterec->datetzone = 0;
+	if (extended) {
+		daterec->datesecfrac = dr.decimicrosecond / 10u;
+		daterec->datesecprec = 1000000;
+	}
+
+	return CS_SUCCEED;
+}
+
 CS_RETCODE
 cs_dt_crack(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
 {
-TDS_DATETIME *dt;
-TDS_DATETIME4 *dt4;
-time_t tmp_secs_from_epoch;
-struct tm *t;
-TDSDATEREC dr;
+	tdsdump_log(TDS_DBG_FUNC, "cs_dt_crack(%p, %d, %p, %p)\n", ctx, datetype, dateval, daterec);
 
-	if (datetype == CS_DATETIME_TYPE) {
-		dt = (TDS_DATETIME *) dateval;
-		tds_datecrack(SYBDATETIME, dt, &dr);
-	} else if (datetype == CS_DATETIME4_TYPE) {
-		dt4 = (TDS_DATETIME4 *) dateval;
-		tds_datecrack(SYBDATETIME4, dt4, &dr);
-	} else {
-		return CS_FAIL;
-	}
-	t = (struct tm *) gmtime(&tmp_secs_from_epoch);
-	daterec->dateyear = dr.year;
-	daterec->datemonth = dr.month;
-	daterec->datedmonth = dr.day;
-	daterec->datedyear = dr.dayofyear;
-	daterec->datedweek = dr.weekday;
-	daterec->datehour = dr.hour;
-	daterec->dateminute = dr.minute;
-	daterec->datesecond = dr.second;
-	daterec->datemsecond = dr.millisecond;
-	daterec->datetzone = 0;
-
-	return CS_SUCCEED;
+	if (datetype != CS_BIGDATETIME_TYPE && datetype != CS_BIGTIME_TYPE)
+		return cs_dt_crack_v2(ctx, datetype, dateval, daterec);
+	return CS_FAIL;
 }
 
 CS_RETCODE
@@ -857,6 +903,8 @@ cs_loc_alloc(CS_CONTEXT * ctx, CS_LOCALE ** locptr)
 {
 	CS_LOCALE *tds_csloc;
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_loc_alloc(%p, %p)\n", ctx, locptr);
+
 	tds_csloc = _cs_locale_alloc();
 	if (!tds_csloc)
 		return CS_FAIL;
@@ -868,6 +916,8 @@ cs_loc_alloc(CS_CONTEXT * ctx, CS_LOCALE ** locptr)
 CS_RETCODE
 cs_loc_drop(CS_CONTEXT * ctx, CS_LOCALE * locale)
 {
+	tdsdump_log(TDS_DBG_FUNC, "cs_loc_drop(%p, %p)\n", ctx, locale);
+
 	if (!locale)
 		return CS_FAIL;
 
@@ -880,6 +930,8 @@ cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_V
 {
 	CS_RETCODE code = CS_FAIL;
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_locale(%p, %d, %p, %d, %p, %d, %p)\n", ctx, action, locale, type, buffer, buflen, outlen);
+
 	if (action == CS_SET) {
 		switch (type) {
 		case CS_LC_ALL:
@@ -895,12 +947,10 @@ cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_V
 			}
 			
 			free(locale->charset);
-			locale->charset = (char *)malloc(buflen + 1);
+			locale->charset = tds_strndup(buffer, buflen);
 			if (!locale->charset)
 				break;
 
-			strncpy(locale->charset, (char *)buffer, buflen);
-			locale->charset[buflen] = '\0';
 			code = CS_SUCCEED;
 			break;
 
@@ -910,12 +960,10 @@ cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_V
 			}
 			
 			free(locale->language);
-			locale->language = (char *)malloc(buflen + 1);
+			locale->language = tds_strndup(buffer, buflen);
 			if (!locale->language)
 				break;
 
-			strncpy(locale->language, (char *)buffer, buflen);
-			locale->language[buflen] = '\0';
 			code = CS_SUCCEED;
 			break;
 
@@ -940,21 +988,15 @@ cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_V
 			}
 			if (i) {
 				free(locale->language);
-				locale->language = (char *)malloc(i + 1);
+				locale->language = tds_strndup(b, i);
 				if (!locale->language)
 					break;
-
-				strncpy(locale->language, b, i);
-				locale->language[i] = '\0';
 			}
 			if (i != (buflen - 1)) {
 				free(locale->charset);
-				locale->charset = (char *)malloc(buflen - i);
+				locale->charset = tds_strndup(b + i + 1, buflen - i - 1);
 				if (!locale->charset)
 					break;
-				
-				strncpy(locale->charset, b + i + 1, buflen - i - 1);
-				locale->charset[buflen - i - 1] = '\0';
 			}
 			code = CS_SUCCEED;
 			break;
@@ -967,12 +1009,10 @@ cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_V
 			}
 			
 			free(locale->collate);
-			locale->collate = (char *)malloc(buflen + 1);
+			locale->collate = tds_strndup(buffer, buflen);
 			if (!locale->collate)
 				break;
 
-			strncpy(locale->collate, (char *)buffer, buflen);
-			locale->collate[buflen] = '\0';
 			code = CS_SUCCEED;
 			break;
 		*/
@@ -1062,6 +1102,9 @@ CS_RETCODE
 cs_dt_info(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_INT item, CS_VOID * buffer, CS_INT buflen,
 	   CS_INT * outlen)
 {
+	tdsdump_log(TDS_DBG_FUNC, "cs_dt_info(%p, %d, %p, %d, %d, %p, %d, %p)\n", 
+				ctx, action, locale, type, item, buffer, buflen, outlen);
+
 	if (action == CS_SET) {
 		switch (type) {
 		case CS_DT_CONVFMT:
@@ -1076,18 +1119,24 @@ cs_strbuild(CS_CONTEXT * ctx, CS_CHAR * buffer, CS_INT buflen, CS_INT * resultle
 	    CS_CHAR * formats, CS_INT formatlen, ...)
 {
 	va_list ap;
-	CS_RETCODE rc;
+	TDSRET rc;
+
+	tdsdump_log(TDS_DBG_FUNC, "cs_strbuild(%p, %p, %d, %p, %p, %d, %p, %d)\n", 
+				ctx, buffer, buflen, resultlen, text, textlen, formats, formatlen);
 
 	va_start(ap, formatlen);
 	rc = tds_vstrbuild(buffer, buflen, resultlen, text, textlen, formats, formatlen, ap);
 	va_end(ap);
-	return rc;
+
+	return TDS_SUCCEED(rc) ? CS_SUCCEED : CS_FAIL;
 }
 
 CS_RETCODE
 cs_calc(CS_CONTEXT * ctx, CS_INT op, CS_INT datatype, CS_VOID * var1, CS_VOID * var2, CS_VOID * dest)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_calc(%p, %d, %d, %p, %p, %p)\n", ctx, op, datatype, var1, var2, dest);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_calc()\n");
 	return CS_FAIL;
 }
@@ -1096,6 +1145,8 @@ CS_RETCODE
 cs_cmp(CS_CONTEXT * ctx, CS_INT datatype, CS_VOID * var1, CS_VOID * var2, CS_INT * result)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_cmp(%p, %d, %p, %p, %p)\n", ctx, datatype, var1, var2, result);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_cmp()\n");
 	return CS_FAIL;
 }
@@ -1104,6 +1155,8 @@ CS_RETCODE
 cs_conv_mult(CS_CONTEXT * ctx, CS_LOCALE * srcloc, CS_LOCALE * destloc, CS_INT * conv_multiplier)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_conv_mult(%p, %p, %p, %p)\n", ctx, srcloc, destloc, conv_multiplier);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_conv_mult()\n");
 	return CS_FAIL;
 }
@@ -1111,6 +1164,8 @@ cs_conv_mult(CS_CONTEXT * ctx, CS_LOCALE * srcloc, CS_LOCALE * destloc, CS_INT *
 CS_RETCODE
 cs_diag(CS_CONTEXT * ctx, CS_INT operation, CS_INT type, CS_INT idx, CS_VOID * buffer)
 {
+	tdsdump_log(TDS_DBG_FUNC, "cs_diag(%p, %d, %d, %d, %p)\n", ctx, operation, type, idx, buffer);
+
 	switch (operation) {
 
 		case CS_INIT:
@@ -1169,6 +1224,9 @@ cs_manage_convert(CS_CONTEXT * ctx, CS_INT action, CS_INT srctype, CS_CHAR * src
 		  CS_CHAR * destname, CS_INT destnamelen, CS_INT * conv_multiplier, CS_CONV_FUNC * func)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_manage_convert(%p, %d, %d, %p, %d, %d, %p, %d, %p, %p)\n", 
+				ctx, action, srctype, srcname, srcnamelen, desttype, destname, destnamelen, conv_multiplier, func);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_manage_convert()\n");
 	return CS_FAIL;
 }
@@ -1177,6 +1235,8 @@ CS_RETCODE
 cs_objects(CS_CONTEXT * ctx, CS_INT action, CS_OBJNAME * objname, CS_OBJDATA * objdata)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_objects(%p, %d, %p, %p)\n", ctx, action, objname, objdata);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_objects()\n");
 	return CS_FAIL;
 }
@@ -1185,6 +1245,8 @@ CS_RETCODE
 cs_set_convert(CS_CONTEXT * ctx, CS_INT action, CS_INT srctype, CS_INT desttype, CS_CONV_FUNC * func)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_set_convert(%p, %d, %d, %d, %p)\n", ctx, action, srctype, desttype, func);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_set_convert()\n");
 	return CS_FAIL;
 }
@@ -1193,6 +1255,8 @@ CS_RETCODE
 cs_setnull(CS_CONTEXT * ctx, CS_DATAFMT * datafmt, CS_VOID * buffer, CS_INT buflen)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_setnull(%p, %p, %p, %d)\n", ctx, datafmt, buffer, buflen);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_setnull()\n");
 	return CS_FAIL;
 }
@@ -1202,6 +1266,9 @@ cs_strcmp(CS_CONTEXT * ctx, CS_LOCALE * locale, CS_INT type, CS_CHAR * str1, CS_
 	  CS_INT * result)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_strcmp(%p, %p, %d, %p, %d, %p, %d, %p)\n", 
+					ctx, locale, type, str1, len1, str2, len2, result);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_strcmp()\n");
 	return CS_FAIL;
 }
@@ -1210,6 +1277,8 @@ CS_RETCODE
 cs_time(CS_CONTEXT * ctx, CS_LOCALE * locale, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen, CS_DATEREC * daterec)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_time(%p, %p, %p, %d, %p, %p)\n", ctx, locale, buffer, buflen, outlen, daterec);
+
 	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_time()\n");
 	return CS_FAIL;
 }
@@ -1218,6 +1287,8 @@ CS_RETCODE
 cs_will_convert(CS_CONTEXT * ctx, CS_INT srctype, CS_INT desttype, CS_BOOL * result)
 {
 
+	tdsdump_log(TDS_DBG_FUNC, "cs_will_convert(%p, %d, %d, %p)\n", ctx, srctype, desttype, result);
+
 	*result = (tds_willconvert(srctype, desttype) ? CS_TRUE : CS_FALSE);
 	return CS_SUCCEED;
 }
@@ -1225,9 +1296,10 @@ cs_will_convert(CS_CONTEXT * ctx, CS_INT srctype, CS_INT desttype, CS_BOOL * res
 static CS_INT 
 cs_diag_storemsg(CS_CONTEXT *context, CS_CLIENTMSG *message)
 {
+	struct cs_diag_msg **curptr;
+	CS_INT msg_count = 0;
 
-struct cs_diag_msg **curptr;
-CS_INT msg_count = 0;
+	tdsdump_log(TDS_DBG_FUNC, "cs_diag_storemsg(%p, %p)\n", context, message);
 
 	curptr = &(context->msgstore);
 
@@ -1247,12 +1319,12 @@ CS_INT msg_count = 0;
 		return CS_FAIL;
 	}
 
-	*curptr = (struct cs_diag_msg *) malloc(sizeof(struct cs_diag_msg));
+	*curptr = tds_new(struct cs_diag_msg, 1);
 	if (*curptr == NULL) { 
 		return CS_FAIL;
 	} else {
 		(*curptr)->next = NULL;
-		(*curptr)->msg  = malloc(sizeof(CS_CLIENTMSG));
+		(*curptr)->msg  = tds_new(CS_CLIENTMSG, 1);
 		if ((*curptr)->msg == NULL) {
 			return CS_FAIL;
 		} else {
@@ -1266,9 +1338,10 @@ CS_INT msg_count = 0;
 static CS_INT 
 cs_diag_getmsg(CS_CONTEXT *context, CS_INT idx, CS_CLIENTMSG *message)
 {
+	struct cs_diag_msg *curptr;
+	CS_INT msg_count = 0, msg_found = 0;
 
-struct cs_diag_msg *curptr;
-CS_INT msg_count = 0, msg_found = 0;
+	tdsdump_log(TDS_DBG_FUNC, "cs_diag_getmsg(%p, %d, %p)\n", context, idx, message);
 
 	curptr = context->msgstore;
 
@@ -1294,8 +1367,9 @@ CS_INT msg_count = 0, msg_found = 0;
 static CS_INT 
 cs_diag_clearmsg(CS_CONTEXT *context, CS_INT type)
 {
+	struct cs_diag_msg *curptr, *freeptr;
 
-struct cs_diag_msg *curptr, *freeptr;
+	tdsdump_log(TDS_DBG_FUNC, "cs_diag_clearmsg(%p, %d)\n", context, type);
 
 	curptr = context->msgstore;
 	context->msgstore = NULL;
@@ -1312,8 +1386,10 @@ struct cs_diag_msg *curptr, *freeptr;
 static CS_INT 
 cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count)
 {
-struct cs_diag_msg *curptr;
-CS_INT msg_count = 0;
+	struct cs_diag_msg *curptr;
+	CS_INT msg_count = 0;
+
+	tdsdump_log(TDS_DBG_FUNC, "cs_diag_countmsg(%p, %p)\n", context, count);
 
 	curptr = context->msgstore;
 
@@ -1324,3 +1400,40 @@ CS_INT msg_count = 0;
 	*count = msg_count;
 	return CS_SUCCEED;
 }
+
+/**
+ * Try to convert to a type we can handle
+ */
+int
+_cs_convert_not_client(CS_CONTEXT *ctx, TDSCOLUMN *curcol, CONV_RESULT *convert_buffer, unsigned char **p_src)
+{
+	int ct_type;
+	TDS_SERVER_TYPE desttype;
+
+	switch (curcol->column_type) {
+	case SYBMSDATE:
+		desttype = SYBDATE;
+		ct_type = CS_DATE_TYPE;
+		break;
+	case SYBMSTIME:
+		desttype = SYB5BIGTIME;
+		ct_type = CS_BIGTIME_TYPE;
+		break;
+	case SYBMSDATETIME2:
+	case SYBMSDATETIMEOFFSET:
+		desttype = SYB5BIGDATETIME;
+		ct_type = CS_BIGDATETIME_TYPE;
+		break;
+	default:
+		return CS_ILLEGAL_TYPE;
+	}
+
+	if (convert_buffer) {
+		int len = tds_convert(ctx->tds_ctx, curcol->column_type, (TDS_CHAR *) *p_src,
+				      curcol->column_cur_size, desttype, convert_buffer);
+		if (len < 0)
+			return CS_ILLEGAL_TYPE; // TODO _csclient_msg ??
+		*p_src = (unsigned char *) convert_buffer;
+	}
+	return ct_type;
+}
