@@ -1,6 +1,4 @@
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -16,11 +14,8 @@
 #include <ctpublic.h>
 #include "common.h"
 
-static char software_version[] = "$Id: lang_ct_param.c,v 1.7 2006/12/26 14:56:18 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 static const char *query =
-	"insert into #ctparam_lang (name,age,cost,bdate,fval) values (@in1, @in2, @moneyval, @dateval, @floatval)";
+	"insert into #ctparam_lang (name,name2,age,cost,bdate,fval) values (@in1, @in2, @in3, @moneyval, @dateval, @floatval)";
 
 CS_RETCODE ex_servermsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_SERVERMSG * errmsg);
 CS_RETCODE ex_clientmsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_CLIENTMSG * errmsg);
@@ -44,7 +39,7 @@ main(int argc, char *argv[])
 	if (argc > 1 && (0 == strcmp(argv[1], "-v")))
 		verbose = 1;
 
-	fprintf(stdout, "%s: submit language query with variables using ct_param \n", __FILE__);
+	printf("%s: submit language query with variables using ct_param\n", __FILE__);
 	if (verbose) {
 		fprintf(stdout, "Trying login\n");
 	}
@@ -59,7 +54,7 @@ main(int argc, char *argv[])
 	ct_callback(ctx, NULL, CS_SET, CS_SERVERMSG_CB, (CS_VOID *) ex_servermsg_cb);
 
 	strcpy(cmdbuf, "create table #ctparam_lang (id numeric identity not null, \
-		name varchar(30), age int, cost money, bdate datetime, fval float) ");
+		name varchar(30), name2 varchar(20), age int, cost money, bdate datetime, fval float) ");
 
 	ret = run_command(cmd, cmdbuf);
 
@@ -74,7 +69,7 @@ main(int argc, char *argv[])
 	/* if worked, test by position */
 	if (0 == errCode)
 		errCode = insert_test(conn, cmd, 0);
-	query = "insert into #ctparam_lang (name,age,cost,bdate,fval) values (?, ?, ?, ?, ?)";
+	query = "insert into #ctparam_lang (name,name2,age,cost,bdate,fval) values (?, ?, ?, ?, ?, ?)";
 	if (0 == errCode)
 		errCode = insert_test(conn, cmd, 0);
 
@@ -111,7 +106,10 @@ insert_test(CS_CONNECTION *conn, CS_COMMAND *cmd, int useNames)
 	CS_DATEREC datevar;
 	char moneystring[10];
 	char dummy_name[30];
+	char dummy_name2[20];
 	CS_INT destlen;
+	CS_INT rowsAffected = -1;
+	int rows_found;
 
 	/* clear table */
 	run_command(cmd, "delete #ctparam_lang");
@@ -123,6 +121,7 @@ insert_test(CS_CONNECTION *conn, CS_COMMAND *cmd, int useNames)
 	intvar = 2;
 	floatvar = 0.12;
 	strcpy(dummy_name, "joe blow");
+	strcpy(dummy_name2, "");
 	strcpy(moneystring, "300.90");
 
 	/*
@@ -200,6 +199,21 @@ insert_test(CS_CONNECTION *conn, CS_COMMAND *cmd, int useNames)
 		strcpy(datafmt.name, "@in2");
 	else
 		datafmt.name[0] = 0;
+	datafmt.maxlength = 255;
+	datafmt.namelen = CS_NULLTERM;
+	datafmt.datatype = CS_CHAR_TYPE;
+	datafmt.status = CS_INPUTVALUE;
+
+	ret = ct_param(cmd, &datafmt, dummy_name2, strlen(dummy_name2), 0);
+	if (CS_SUCCEED != ret) {
+		fprintf(stderr, "ct_param(char) failed\n");
+		return 1;
+	}
+
+	if (useNames)
+		strcpy(datafmt.name, "@in3");
+	else
+		datafmt.name[0] = 0;
 	datafmt.namelen = CS_NULLTERM;
 	datafmt.datatype = CS_INT_TYPE;
 	datafmt.status = CS_INPUTVALUE;
@@ -277,6 +291,8 @@ insert_test(CS_CONNECTION *conn, CS_COMMAND *cmd, int useNames)
 				ct_res_info(cmd, CS_ROW_COUNT, &rowsAffected, CS_UNUSED, NULL);
 				if (1 != rowsAffected) 
 					fprintf(stderr, "insert touched %d rows instead of 1\n", rowsAffected);
+				else
+					rows_found = 1;
 			}
 			break;
 
@@ -304,11 +320,18 @@ insert_test(CS_CONNECTION *conn, CS_COMMAND *cmd, int useNames)
 			return 1;
 		}
 	}
-	if (ret != CS_END_RESULTS)
+	if (ret != CS_END_RESULTS) {
 		fprintf(stderr, "ct_results returned unexpected result %d.\n", (int) ret);
+		exit(1);
+	}
+
+	if (!rows_found) {
+		fprintf(stderr, "expected 1 rows inserted, inserted %d.\n", (int) rowsAffected);
+		exit(1);
+	}
 
 	/* test row inserted */
-	ret = run_command(cmd, "if not exists(select * from #ctparam_lang) select 1");
+	ret = run_command(cmd, "if not exists(select * from #ctparam_lang where name = 'joe blow' and name2 is not null and age = 2) select 1");
 	if (ret != CS_SUCCEED) {
 		fprintf(stderr, "check row inserted failed\n");
 		exit(1);
