@@ -8,18 +8,7 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#if TIME_WITH_SYS_TIME
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# endif
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <freetds/time.h>
 
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -37,8 +26,11 @@
 #include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
 
-static char software_version[] = "$Id: hang.c,v 1.3 2008/11/25 22:58:29 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#if HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif /* HAVE_NET_INET_IN_H */
+
+#include <freetds/sysdep_private.h>
 
 char *UNITTEST;
 
@@ -47,19 +39,25 @@ char *UNITTEST;
 static int end_socket = -1;
 
 static int
-shutdown_last_socket(void)
+shutdown_socket(DBPROCESS *dbproc)
 {
-	int max_socket = -1, i;
-	int sockets[2];
+	union {
+		struct sockaddr sa;
+		char data[256];
+	} u;
+	SOCKLEN_T addrlen;
+	struct stat file_stat;
+	TDS_SYS_SOCKET sockets[2];
 
-	for (i = 3; i < 1024; ++i) {
-		struct stat file_stat;
-		if (fstat(i, &file_stat))
-			continue;
-		if ((file_stat.st_mode & S_IFSOCK) == S_IFSOCK)
-			max_socket = i;
-	}
-	if (max_socket < 0)
+	TDS_SYS_SOCKET socket = DBIOWDESC(dbproc);
+
+	if (fstat(socket, &file_stat))
+		return 0;
+	if ((file_stat.st_mode & S_IFSOCK) != S_IFSOCK)
+		return 0;
+
+	addrlen = sizeof(u);
+	if (tds_getsockname(socket, &u.sa, &addrlen) < 0 || (u.sa.sa_family != AF_INET && u.sa.sa_family != AF_INET6))
 		return 0;
 
 	/* replace socket with a new one */
@@ -67,8 +65,8 @@ shutdown_last_socket(void)
 		return 0;
 
 	/* substitute socket */
-	close(max_socket);
-	dup2(sockets[0], max_socket);
+	close(socket);
+	dup2(sockets[0], socket);
 
 	/* close connection */
 	close(sockets[0]);
@@ -115,7 +113,7 @@ test(int close_socket)
 		return 1;
 	}
 
-	if (!shutdown_last_socket()) {
+	if (!shutdown_socket(dbproc)) {
 		fprintf(stderr, "Error shutting down connection\n");
 		return 1;
 	}
