@@ -5,11 +5,6 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0009.c,v 1.14 2006/07/06 12:48:16 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-
-
 int
 main(int argc, char **argv)
 {
@@ -23,19 +18,16 @@ main(int argc, char **argv)
 
 	read_login_info(argc, argv);
 
-	fprintf(stdout, "Start\n");
-	add_bread_crumb();
+	fprintf(stdout, "Starting %s\n", argv[0]);
 
 	/* Fortify_EnterScope(); */
 	dbinit();
 
-	add_bread_crumb();
 	dberrhandle(syb_err_handler);
 	dbmsghandle(syb_msg_handler);
 
 	fprintf(stdout, "About to logon\n");
 
-	add_bread_crumb();
 	login = dblogin();
 	DBSETLPWD(login, PASSWORD);
 	DBSETLUSER(login, USER);
@@ -44,35 +36,31 @@ main(int argc, char **argv)
 
 	fprintf(stdout, "About to open\n");
 
-	add_bread_crumb();
 	dbproc = dbopen(login, SERVER);
 	if (strlen(DATABASE))
 		dbuse(dbproc, DATABASE);
-	add_bread_crumb();
 	dbloginfree(login);
-	add_bread_crumb();
 
 #ifdef MICROSOFT_DBLIB
 	dbsetopt(dbproc, DBBUFFER, "100");
 #else
 	dbsetopt(dbproc, DBBUFFER, "100", 0);
 #endif
-	add_bread_crumb();
 
 	fprintf(stdout, "creating table\n");
-	dbcmd(dbproc, "create table #dblib0009 (i int not null, s char(10) not null)");
+	sql_cmd(dbproc);
 	dbsqlexec(dbproc);
 	while (dbresults(dbproc) != NO_MORE_RESULTS) {
 		/* nop */
 	}
 
 	fprintf(stdout, "insert\n");
-	dbcmd(dbproc, "insert into #dblib0009 values (1, 'abcdef')");
+	sql_cmd(dbproc);
 	dbsqlexec(dbproc);
 	while (dbresults(dbproc) != NO_MORE_RESULTS) {
 		/* nop */
 	}
-	dbcmd(dbproc, "insert into #dblib0009 values (2, 'abc')");
+	sql_cmd(dbproc);
 	dbsqlexec(dbproc);
 	while (dbresults(dbproc) != NO_MORE_RESULTS) {
 		/* nop */
@@ -80,32 +68,19 @@ main(int argc, char **argv)
 
 
 	fprintf(stdout, "select\n");
-	dbcmd(dbproc, "select * from #dblib0009 order by i");
+	sql_cmd(dbproc);
 	dbsqlexec(dbproc);
-	add_bread_crumb();
-
 
 	if (dbresults(dbproc) != SUCCEED) {
-		add_bread_crumb();
 		fprintf(stdout, "Was expecting a result set.");
 		exit(1);
 	}
-	add_bread_crumb();
 
-	for (i = 1; i <= dbnumcols(dbproc); i++) {
-		add_bread_crumb();
+	for (i = 1; i <= dbnumcols(dbproc); i++)
 		printf("col %d is %s\n", i, dbcolname(dbproc, i));
-		add_bread_crumb();
-	}
 
-	add_bread_crumb();
 	dbbind(dbproc, 1, INTBIND, -1, (BYTE *) & testint);
-	add_bread_crumb();
 	dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr);
-	add_bread_crumb();
-
-	add_bread_crumb();
-
 
 	if (REG_ROW != dbnextrow(dbproc)) {
 		fprintf(stderr, "dblib failed for %s\n", __FILE__);
@@ -127,11 +102,8 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	add_bread_crumb();
 	dbexit();
-	add_bread_crumb();
 
 	printf("dblib passed for %s\n", __FILE__);
-	free_bread_crumb();
 	return 0;
 }
