@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
- * Copyright (C) 2005-2010  Frediano Ziglio
+ * Copyright (C) 2005-2011  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -29,16 +27,10 @@
 #include <ctype.h>
 #include <sys/stat.h>
 
-#include "tdsodbc.h"
-#include "tdsstring.h"
+#include <freetds/odbc.h>
+#include <freetds/string.h>
 #include "replacements.h"
 
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: connectparams.c,v 1.88 2010/11/09 15:46:42 freddy77 Exp $");
-
 #define ODBC_PARAM(p) static const char odbc_param_##p[] = #p;
 ODBC_PARAM_LIST
 #undef ODBC_PARAM
@@ -108,13 +100,12 @@ static int SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR
 #endif
 
 static int
-parse_server(TDS_ERRS *errs, char *server, TDSCONNECTION * connection)
+parse_server(TDS_ERRS *errs, char *server, TDSLOGIN * login)
 {
-	char ip[64];
 	char *p = (char *) strchr(server, '\\');
 
 	if (p) {
-		if (!tds_dstr_copy(&connection->instance_name, p+1)) {
+		if (!tds_dstr_copy(&login->instance_name, p+1)) {
 			odbc_errs_add(errs, "HY001", NULL);
 			return 0;
 		}
@@ -122,18 +113,16 @@ parse_server(TDS_ERRS *errs, char *server, TDSCONNECTION * connection)
 	} else {
 		p = (char *) strchr(server, ',');
 		if (p && atoi(p+1) > 0) {
-			connection->port = atoi(p+1);
+			login->port = atoi(p+1);
 			*p = 0;
 		}
 	}
 
-	if (tds_lookup_host(server, ip) == TDS_SUCCEED)
-		tds_dstr_copy(&connection->server_host_name, server);
-
-	if (!tds_dstr_copy(&connection->ip_addr, ip)) {
-		odbc_errs_add(errs, "HY001", NULL);
-		return 0;
-	}
+	if (TDS_SUCCEED(tds_lookup_host_set(server, &login->ip_addrs)))
+		if (!tds_dstr_copy(&login->server_host_name, server)) {
+			odbc_errs_add(errs, "HY001", NULL);
+			return 0;
+		}
 
 	return 1;
 }
@@ -148,11 +137,11 @@ myGetPrivateProfileString(const char *DSN, const char *key, char *buf)
 /** 
  * Read connection information from given DSN
  * @param DSN           DSN name
- * @param connection    where to store connection info
+ * @param login    where to store connection info
  * @return 1 if success 0 otherwhise
  */
 int
-odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSCONNECTION * connection)
+odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 {
 	char tmp[FILENAME_MAX];
 	int freetds_conf_less = 1;
@@ -160,8 +149,11 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSCONNECTION * connection)
 	/* use old servername */
 	if (myGetPrivateProfileString(DSN, odbc_param_Servername, tmp) > 0) {
 		freetds_conf_less = 0;
-		tds_dstr_copy(&connection->server_name, tmp);
-		tds_read_conf_file(connection, tmp);
+		if (!tds_dstr_copy(&login->server_name, tmp)) {
+			odbc_errs_add(errs, "HY001", NULL);
+			return 0;
+		}
+		tds_read_conf_file(login, tmp);
 		if (myGetPrivateProfileString(DSN, odbc_param_Server, tmp) > 0) {
 			odbc_errs_add(errs, "HY000", "You cannot specify both SERVERNAME and SERVER");
 			return 0;
@@ -179,76 +171,109 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSCONNECTION * connection)
 		if (myGetPrivateProfileString(DSN, odbc_param_Address, tmp) > 0) {
 			address_specified = 1;
 			/* TODO parse like MS */
-			tds_lookup_host(tmp, tmp);
-			tds_dstr_copy(&connection->ip_addr, tmp);
+
+			if (TDS_FAILED(tds_lookup_host_set(tmp, &login->ip_addrs))) {
+				odbc_errs_add(errs, "HY000", "Error parsing ADDRESS attribute");
+				return 0;
+			}
 		}
 		if (myGetPrivateProfileString(DSN, odbc_param_Server, tmp) > 0) {
-			tds_dstr_copy(&connection->server_name, tmp);
+			if (!tds_dstr_copy(&login->server_name, tmp)) {
+				odbc_errs_add(errs, "HY001", NULL);
+				return 0;
+			}
 			if (!address_specified) {
-				if (!parse_server(errs, tmp, connection))
+				if (!parse_server(errs, tmp, login))
 					return 0;
 			}
 		}
 	}
 
 	if (myGetPrivateProfileString(DSN, odbc_param_Port, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_PORT, tmp, connection);
+		tds_parse_conf_section(TDS_STR_PORT, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_TDS_Version, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_VERSION, tmp, connection);
+		tds_parse_conf_section(TDS_STR_VERSION, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_Language, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_LANGUAGE, tmp, connection);
+		tds_parse_conf_section(TDS_STR_LANGUAGE, tmp, login);
 
-	if (tds_dstr_isempty(&connection->database)
+	if (tds_dstr_isempty(&login->database)
 	    && myGetPrivateProfileString(DSN, odbc_param_Database, tmp) > 0)
-		tds_dstr_copy(&connection->database, tmp);
+		if (!tds_dstr_copy(&login->database, tmp)) {
+			odbc_errs_add(errs, "HY001", NULL);
+			return 0;
+		}
 
 	if (myGetPrivateProfileString(DSN, odbc_param_TextSize, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_TEXTSZ, tmp, connection);
+		tds_parse_conf_section(TDS_STR_TEXTSZ, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_PacketSize, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_BLKSZ, tmp, connection);
+		tds_parse_conf_section(TDS_STR_BLKSZ, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_ClientCharset, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_CLCHARSET, tmp, connection);
+		tds_parse_conf_section(TDS_STR_CLCHARSET, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_DumpFile, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_DUMPFILE, tmp, connection);
+		tds_parse_conf_section(TDS_STR_DUMPFILE, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_DumpFileAppend, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_APPENDMODE, tmp, connection);
+		tds_parse_conf_section(TDS_STR_APPENDMODE, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_DebugFlags, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_DEBUGFLAGS, tmp, connection);
+		tds_parse_conf_section(TDS_STR_DEBUGFLAGS, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_Encryption, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_ENCRYPTION, tmp, connection);
+		tds_parse_conf_section(TDS_STR_ENCRYPTION, tmp, login);
 
 	if (myGetPrivateProfileString(DSN, odbc_param_UseNTLMv2, tmp) > 0)
-		tds_parse_conf_section(TDS_STR_USENTLMV2, tmp, connection);
+		tds_parse_conf_section(TDS_STR_USENTLMV2, tmp, login);
 
-	if (myGetPrivateProfileString(DSN, odbc_param_Trusted_Connection, tmp) > 0 && tds_config_boolean(tmp)) {
-		tds_dstr_copy(&connection->user_name, "");
-		tds_dstr_copy(&connection->password, "");
+	if (myGetPrivateProfileString(DSN, odbc_param_REALM, tmp) > 0)
+		tds_parse_conf_section(TDS_STR_REALM, tmp, login);
+
+	if (myGetPrivateProfileString(DSN, odbc_param_ServerSPN, tmp) > 0)
+		tds_parse_conf_section(TDS_STR_SPN, tmp, login);
+
+	if (myGetPrivateProfileString(DSN, odbc_param_Trusted_Connection, tmp) > 0 && tds_config_boolean(odbc_param_Trusted_Connection, tmp, login)) {
+		tds_dstr_empty(&login->user_name);
+		tds_dstr_empty(&login->password);
 	}
 
+	if (myGetPrivateProfileString(DSN, odbc_param_MARS_Connection, tmp) > 0 && tds_config_boolean(odbc_param_MARS_Connection, tmp, login)) {
+		login->mars = 1;
+	}
+
+	if (myGetPrivateProfileString(DSN, odbc_param_AttachDbFilename, tmp) > 0)
+		tds_parse_conf_section(TDS_STR_DBFILENAME, tmp, login);
+
 	return 1;
 }
 
+/**
+ * Swap two DSTR
+ */
+static void
+odbc_dstr_swap(DSTR *a, DSTR *b)
+{
+	DSTR tmp = *a;
+	*a = *b;
+	*b = tmp;
+}
+
 /** 
- * Parse connection string and fill connection according
+ * Parse connection string and fill login according
  * @param connect_string     connect string
  * @param connect_string_end connect string end (pointer to char past last)
- * @param connection         where to store connection info
+ * @param login         where to store connection info
  * @return 1 if success 0 otherwhise
  */
 int
-odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char *connect_string_end, TDSCONNECTION * connection,
+odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char *connect_string_end, TDSLOGIN * login,
 			  TDS_PARSED_PARAM *parsed_params)
 {
 	const char *p, *end;
-	DSTR *dest_s, value;
+	DSTR *dest_s, value = DSTR_INITIALIZER;
 	enum { CFG_DSN = 1, CFG_SERVER = 2, CFG_SERVERNAME = 4 };
 	unsigned int cfgs = 0;	/* flags for indicate second parse of string */
 	char option[24];
@@ -257,7 +282,6 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 	if (parsed_params)
 		memset(parsed_params, 0, sizeof(*parsed_params)*ODBC_PARAM_SIZE);
 
-	tds_dstr_init(&value);
 	for (p = connect_string; p < connect_string_end && *p;) {
 		int num_param = -1;
 
@@ -314,9 +338,9 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 				return 0;
 			}
 			if (!cfgs) {
-				dest_s = &connection->server_name;
+				dest_s = &login->server_name;
 				/* not that safe cast but works -- freddy77 */
-				if (!parse_server(errs, (char *) tds_dstr_cstr(&value), connection)) {
+				if (!parse_server(errs, (char *) tds_dstr_cstr(&value), login)) {
 					tds_dstr_free(&value);
 					return 0;
 				}
@@ -329,8 +353,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 				return 0;
 			}
 			if (!cfgs) {
-				tds_dstr_dup(&connection->server_name, &value);
-				tds_read_conf_file(connection, tds_dstr_cstr(&value));
+				odbc_dstr_swap(&login->server_name, &value);
+				tds_read_conf_file(login, tds_dstr_cstr(&login->server_name));
 				cfgs = CFG_SERVERNAME;
 				p = connect_string;
 				continue;
@@ -342,7 +366,7 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 				return 0;
 			}
 			if (!cfgs) {
-				if (!odbc_get_dsn_info(errs, tds_dstr_cstr(&value), connection)) {
+				if (!odbc_get_dsn_info(errs, tds_dstr_cstr(&value), login)) {
 					tds_dstr_free(&value);
 					return 0;
 				}
@@ -351,42 +375,66 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 				continue;
 			}
 		} else if (CHK_PARAM(Database)) {
-			dest_s = &connection->database;
+			dest_s = &login->database;
 		} else if (CHK_PARAM(UID)) {
-			dest_s = &connection->user_name;
+			dest_s = &login->user_name;
 		} else if (CHK_PARAM(PWD)) {
-			dest_s = &connection->password;
+			dest_s = &login->password;
 		} else if (CHK_PARAM(APP)) {
-			dest_s = &connection->app_name;
+			dest_s = &login->app_name;
 		} else if (CHK_PARAM(WSID)) {
-			dest_s = &connection->client_host_name;
+			dest_s = &login->client_host_name;
 		} else if (CHK_PARAM(Language)) {
-			tds_parse_conf_section(TDS_STR_LANGUAGE, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_LANGUAGE, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(Port)) {
-			tds_parse_conf_section(TDS_STR_PORT, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_PORT, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(TDS_Version)) {
-			tds_parse_conf_section(TDS_STR_VERSION, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_VERSION, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(TextSize)) {
-			tds_parse_conf_section(TDS_STR_TEXTSZ, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_TEXTSZ, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(PacketSize)) {
-			tds_parse_conf_section(TDS_STR_BLKSZ, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_BLKSZ, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(ClientCharset)) {
-			tds_parse_conf_section(TDS_STR_CLCHARSET, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_CLCHARSET, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(DumpFile)) {
-			tds_parse_conf_section(TDS_STR_DUMPFILE, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_DUMPFILE, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(DumpFileAppend)) {
-			tds_parse_conf_section(TDS_STR_APPENDMODE, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_APPENDMODE, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(DebugFlags)) {
-			tds_parse_conf_section(TDS_STR_DEBUGFLAGS, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_DEBUGFLAGS, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(Encryption)) {
-			tds_parse_conf_section(TDS_STR_ENCRYPTION, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_ENCRYPTION, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(UseNTLMv2)) {
-			tds_parse_conf_section(TDS_STR_USENTLMV2, tds_dstr_cstr(&value), connection);
+			tds_parse_conf_section(TDS_STR_USENTLMV2, tds_dstr_cstr(&value), login);
+		} else if (CHK_PARAM(REALM)) {
+			tds_parse_conf_section(TDS_STR_REALM, tds_dstr_cstr(&value), login);
+		} else if (CHK_PARAM(ServerSPN)) {
+			tds_parse_conf_section(TDS_STR_SPN, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(Trusted_Connection)) {
-			trusted = tds_config_boolean(tds_dstr_cstr(&value));
+			trusted = tds_config_boolean(option, tds_dstr_cstr(&value), login);
 			tdsdump_log(TDS_DBG_INFO1, "trusted %s -> %d\n", tds_dstr_cstr(&value), trusted);
 			num_param = -1;
 			/* TODO odbc_param_Address field */
+		} else if (CHK_PARAM(MARS_Connection)) {
+			if (tds_config_boolean(option, tds_dstr_cstr(&value), login))
+				login->mars = 1;
+		} else if (CHK_PARAM(AttachDbFilename)) {
+			dest_s = &login->db_filename;
+		} else if (CHK_PARAM(ApplicationIntent)) {
+			const char *readonly_intent;
+
+			if (strcasecmp(tds_dstr_cstr(&value), "ReadOnly") == 0) {
+				readonly_intent = "yes";
+			} else if (strcasecmp(tds_dstr_cstr(&value), "ReadWrite") == 0) {
+				readonly_intent = "no";
+			} else {
+				tdsdump_log(TDS_DBG_ERROR, "Invalid ApplicationIntent %s\n", tds_dstr_cstr(&value));
+				tds_dstr_free(&value);
+				return 0;
+			}
+
+			tds_parse_conf_section(TDS_STR_READONLY_INTENT, readonly_intent, login);
+			tdsdump_log(TDS_DBG_INFO1, "Application Intent %s\n", readonly_intent);
 		}
 
 		if (num_param >= 0 && parsed_params) {
@@ -395,11 +443,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 		}
 
 		/* copy to destination */
-		if (dest_s) {
-			DSTR tmp = *dest_s;
-			*dest_s = value;
-			value = tmp;
-		}
+		if (dest_s)
+			odbc_dstr_swap(dest_s, &value);
 
 		p = end;
 		/* handle "" ";.." "};.." cases */
@@ -417,8 +462,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 			parsed_params[ODBC_PARAM_UID].p = NULL;
 			parsed_params[ODBC_PARAM_PWD].p = NULL;
 		}
-		tds_dstr_copy(&connection->user_name, "");
-		tds_dstr_copy(&connection->password, "");
+		tds_dstr_empty(&login->user_name);
+		tds_dstr_empty(&login->password);
 	}
 
 	tds_dstr_free(&value);
@@ -440,7 +485,7 @@ odbc_build_connect_string(TDS_ERRS *errs, TDS_PARSED_PARAM *params, char **out)
 	}
 
 	/* allocate */
-	p = (char*) malloc(len);
+	p = tds_new(char, len);
 	if (!p) {
 		odbc_errs_add(errs, "HY001", NULL);
 		return 0;
@@ -479,7 +524,7 @@ tdoParseProfile(const char *option, const char *value, void *param)
 	ProfileParam *p = (ProfileParam *) param;
 
 	if (strcasecmp(p->entry, option) == 0) {
-		tds_strlcpy(p->buffer, value, p->buffer_len);
+		strlcpy(p->buffer, value, p->buffer_len);
 
 		p->ret_val = strlen(p->buffer);
 		p->found = 1;
@@ -531,7 +576,7 @@ SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault
 	tds_read_conf_section(hFile, pszSection, tdoParseProfile, &param);
 
 	if (pszDefault && !param.found) {
-		tds_strlcpy(pRetBuffer, pszDefault, nRetBuffer);
+		strlcpy(pRetBuffer, pszDefault, nRetBuffer);
 
 		param.ret_val = strlen(pRetBuffer);
 	}
@@ -626,6 +671,8 @@ static const char *const aTDSver[] = {
 	"7.0",
 	"7.1",
 	"7.2",
+	"7.3",
+	"7.4",
 	NULL
 };
 
@@ -669,8 +716,8 @@ definePropertyString(HODBCINSTPROPERTY hLastProperty, const char *name, const ch
 {
 	hLastProperty = addProperty(hLastProperty);
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	tds_strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	tds_strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
+	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
+	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup(comment);
 	return hLastProperty;
 }
@@ -682,8 +729,8 @@ definePropertyBoolean(HODBCINSTPROPERTY hLastProperty, const char *name, const c
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
 	hLastProperty->aPromptData = malloc(sizeof(aBoolean));
 	memcpy(hLastProperty->aPromptData, aBoolean, sizeof(aBoolean));
-	tds_strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	tds_strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
+	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
+	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup(comment);
 	return hLastProperty;
 }
@@ -693,8 +740,8 @@ definePropertyHidden(HODBCINSTPROPERTY hLastProperty, const char *name, const ch
 {
 	hLastProperty = addProperty(hLastProperty);
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_HIDDEN;
-	tds_strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	tds_strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
+	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
+	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup(comment);
 	return hLastProperty;
 }
@@ -706,8 +753,8 @@ definePropertyList(HODBCINSTPROPERTY hLastProperty, const char *name, const char
 	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
 	hLastProperty->aPromptData = malloc(size);
 	memcpy(hLastProperty->aPromptData, list, size);
-	tds_strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	tds_strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
+	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
+	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
 	hLastProperty->pszHelp = (char *) strdup(comment);
 	return hLastProperty;
 }
@@ -740,7 +787,10 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 		" 5.0 Sybase >= 10.x\n"
 		" 7.0 MSSQL 7\n"
 		" 7.1 MSSQL 2000\n"
-		" 7.2 MSSQL 2005");
+		" 7.2 MSSQL 2005\n"
+		" 7.3 MSSQL 2008\n"
+		" 7.4 MSSQL 2012 or 2014"
+		);
 
 	hLastProperty = definePropertyList(hLastProperty, odbc_param_Language, "us_english", (void*) aLanguage, sizeof(aLanguage),
 		"The default language setting.");
