@@ -6,6 +6,8 @@
 
 #include <assert.h>
 
+#include "replacements.h"
+
 /*
  * Test timeout on prepare
  * It execute a query wait for timeout and then try to issue a new prepare/execute
@@ -14,13 +16,6 @@
  * Test from Ou Liu, cf "Query Time Out", 2006-08-08
  */
 
-static char software_version[] = "$Id: timeout2.c,v 1.9 2010/07/05 09:20:33 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-#if defined(__MINGW32__) || defined(_WIN32)
-#define sleep(s) Sleep((s)*1000)
-#endif
-
 int
 main(int argc, char *argv[])
 {
@@ -37,7 +32,7 @@ main(int argc, char *argv[])
 
 		CHKSetStmtAttr(SQL_ATTR_QUERY_TIMEOUT, (SQLPOINTER) 10, SQL_IS_UINTEGER, "S");
 
-		CHKPrepare((SQLCHAR*) "select * from #timeout", SQL_NTS, "S");
+		CHKPrepare(T("select * from #timeout"), SQL_NTS, "S");
 		CHKExecute("S");
 
 		do {
@@ -47,7 +42,7 @@ main(int argc, char *argv[])
 
 		if (i == 0) {
 			printf("Sleep 15 seconds to test if timeout occurs\n");
-			sleep(15);
+			tds_sleep_s(15);
 		}
 
 		SQLFreeStmt(odbc_stmt, SQL_CLOSE);
@@ -58,5 +53,6 @@ main(int argc, char *argv[])
 
 	odbc_disconnect();
 
+	ODBC_FREE();
 	return 0;
 }
